#!/bin/sh
# update firefox to latest official version
# jamesbond 2015
# MIT license
# L18L internationalized to get also localized version; fixed 2016

export TEXTDOMAIN=get_app

### config
APPTITLE="$(gettext 'Update Firefox')"
WORKDIR=/tmp/update-firefox.$$
LIBDIR=lib64 # lib for 32-bit

# make sure we cleanup after ourselves
trap 'rm -rf $WORKDIR; exit' 0 INT HUP TERM

### generic helpers
# $1-text
msg() {
	[ -z "$DISPLAY" ] && echo "$1" ||
	Xdialog --title "$APPTITLE" --infobox "$1" 0 0 10000
}

# are we root?
check_root() {
	if [ $(id -u) -ne 0 ]; then
		msg "$(gettext 'You must be root.')"
		exit 1
	fi
}

# $1-text, returns true/false
yesno() {
	local p
	if [ -z "$DISPLAY" ]; then
		printf "%s\nType [yes] to continue: " "$1"
		read p
		case "${p,,}" in
			yes) return 0
		esac
		return 1
	else
		Xdialog --title "$APPTITLE" --yesno "$1" 0 0
	fi
}

# $1-text
splash() {
    if [ -z "$DISPLAY" ]; then
        echo "$1"
    else
        if [ "$1" ]; then
            [ $SPID ] && kill $SPID
            Xdialog --title "$APPTITLE" --no-buttons --infobox "$1" 0 0 100000000 &
            SPID=$!
        else
            [ $SPID ] && kill $SPID
            SPID=""
        fi
    fi
}

check_online () {
    if [ ! "$(grep up /sys/class/net/*/operstate)" ]; then
        msg "$gettext 'No connection to internet. Leaving now.'"
        exit
    fi    
}

#$1 #ex:'latest-firefox'
#$2 #a file containing $1 ex:'<html class="windows x86 no-js" lang="en" dir="ltr" data-latest-firefox="43.0.4" data-esr-versions="38.5.2">'
parse_value () {
    local LINE x
    grep -m 1 $1 $2 | tr ' ' '\n' | while read LINE
    do
     case "$LINE" in *${1}*) x=${LINE#*=} ; echo ${x//\"/};; esac; 
    done #ex: 43.0.4

} ; export -f parse_value

### firefox stuff
get_latest_version() {
	# because web scraping is KING!
	#wget -qO - https://www.mozilla.org/en-US/firefox/new/ |
	#sed -n '/firefox-[0-9][^-]*-/ { s/.*firefox-\([0-9][^-]*\)-.*/\1/; p}' |
	#head -n 1
	wget -qO FF_index https://www.mozilla.org/en-US/firefox/all/
	#because containing also esr-versions and "all" full language names
    parse_value latest-firefox FF_index
    #parse_value esr-versions FF_index
} ; export get_latest_version

get_current_version() {
	ls -ld /usr/$LIBDIR/firefox-* 2>/dev/null | tail -n 1 | sed 's/.*firefox-//'
} ; export -f get_current_version

get_path() {
    local FFLANG FOUND NUM LINE x
    read FFLANG < ${FATDOG_STATE_DIR}/language ; FFLANG=${FFLANG%.*}
    #default en_US
#    grep 'os=linux64&amp;lang='${FFLANG%_*} FF_index | cut -d'"' -f 4 > ALL \
# || grep 'os=linux64&amp;lang=en_US' FF_index | cut -d'"' -f 4 > ALL
    #ex:https://download.mozilla.org/?product=firefox-43.0.4-SSL&amp;os=linux64&amp;lang=en-GB

    #no default, always choice:
    grep 'os=linux64&amp;lang=' FF_index | cut -d'"' -f 4 > ALL

    FOUND=$(cat ALL | wc -l)
    NUM=1
    case $FOUND in
     1) : ;;
     *) rm ALL_ 2>/dev/null
        for LINE in $(cat ALL); do
            #[ "${LINE##*-}" = "${FFLANG#*_}" ] && break # ex: US or GB or...
          #  x=$(grep -m 1 "lang=\"${LINE##*=}\"" FF_index | cut -d'>' -f2 | cut -d'<' -f1)

            x=$(grep -m 1 "id=\"${LINE##*=}\"" FF_index | cut -d'"' -f4 | cut -d'"' -f1)

            #broke Xdialog because of japanese, thus:
            #[ "$x" ] || x=$(grep "lang=\"${LINE##*=}\"" FF_index | tail -n 1 | cut -d'>' -f2 | cut -d'<' -f1)

            [ "$x" ] || x=$(grep "id=\"${LINE##*=}\"" FF_index | tail -n 1 | cut -d'"' -f4 | cut -d'"' -f1)

            x="${x#*,}≪${x%,*}" ;  x=${x:1}
            [ "${x:0:7}" = "english" ] && x="${x%≪*}" #fix for crazy mozilla
            echo "$NUM ${x// /…} " >> ALL_
            NUM=$(( NUM + 1 ))
        done
        LC_ALL=C sort -k 2 --field-separator=' ' ALL_ > _ALL_ && mv _ALL_ ALL_ || msg "no sort"
        
        #move some items to top of list
        TOP="Português Norsk Español English"
        TOP="portugu norwegi spanish english"
        for LINE in $TOP; do
            grep ${LINE} ALL_ > _ALL_ ; grep -v ${LINE} ALL_ >> _ALL_
            mv _ALL_ ALL_ ; sync
        done
        #move your locale to top of list
        TOP="$(locale -a | grep -Ev "^C|POSIX|pt|no|es|en" | cut -d'_' -f1 | uniq)"
        for LINE in $TOP; do
            #x=$(grep "lang=\"${LINE}" FF_index | tail -n 1 | cut -d'>' -f2 | cut -d'<' -f1)
            x=$(grep "id=\"${LINE##*=}\"" FF_index | tail -n 1 | cut -d'"' -f4 | cut -d'"' -f1) 
            x="${x#*,}" ;  x=${x:1}
            grep "${x}" ALL_ > _ALL_ ; grep -v "${x}" ALL_ >> _ALL_
            mv _ALL_ ALL_ ; sync
        done
        #choose language or cancel
        Xdialog --title "$APPTITLE" --left --menu "99 Language Name ≪ English Name " 2000 64 0 $(cat ALL_) 2>CHOICE
        [ $? -eq 0 ] || exit
        NUM=$(tail -n 1 CHOICE)
        ;;
    esac
    sed -n ${NUM}p ALL
} ; export -f get_path

# $1-version
package_firefox() {
	splash "$(eval echo `gettext 'Making firefox $1 package, the package will be stored in $HOME'`)"
	mkdir pkg.$$; cd pkg.$$
	
	# extract
	mkdir -p usr/$LIBDIR/ usr/share/pixmaps usr/bin usr/share/applications
	tar -xf ../firefox.tar.bz2 -C usr/$LIBDIR
	
	# setup symlinks, etc
	mv usr/$LIBDIR/firefox usr/$LIBDIR/firefox-$1
	ln -s firefox-$1 usr/$LIBDIR/firefox
	ln -s /usr/$LIBDIR/firefox/firefox usr/bin/firefox
	ln -s /usr/$LIBDIR/firefox-$VERSION/browser/icons/mozicon128.png \
        usr/share/pixmaps/firefox.png &&
    
	# run firefox as spot
	cat << "EOF" >> usr/bin/firefox-spot &&
#!/bin/dash
SPOT_HOME=$(awk -F: '$1=="spot" {print $6}' /etc/passwd)
#{ chown spot $SPOT_HOME/Downloads;
#chown spot /aufs/devsave/Downloads; } 2> /dev/null
cd $SPOT_HOME/Downloads
exec /usr/bin/run-as-spot /usr/bin/firefox "$@"
EOF
	chmod +x usr/bin/firefox-spot &&
	
	# desktop file
	cat << "EOF" > usr/share/applications/firefox.desktop 
[Desktop Entry]
Version=1.0
Name=Firefox Browser
GenericName=Web Browser
Comment=Your web, the way you like it
Exec=firefox-spot %U
Icon=firefox.png
Terminal=false
Type=Application
MimeType=text/html;text/xml;application/xhtml+xml;application/vnd.mozilla.xul+xml;text/mml;x-scheme-handler/http;x-scheme-handler/https;
StartupNotify=false
X-MultipleArgs=false
X-Desktop-File-Install-Version=0.16
Categories=WebBrowser;
Encoding=UTF-8
EOF

	# desc
	mkdir install
	cat << EOF > install/slack-desc
firefox: firefox $1 (Web browser)  
firefox:  
firefox: Official Firefox web browser from Mozilla.
firefox: 
firefox: 
firefox: 
firefox: 
firefox: 
firefox: 
firefox: 
firefox:  
EOF
	# make the pkg
	makepkg -c n -l n $HOME/firefox-$1-$ARCH-official.txz > /dev/null 2>&1
}

# $1-version
install_firefox() {
	splash "$(eval echo `gettext 'Upgrading firefox to $1...'`)"
	removepkg firefox
	installpkg $HOME/firefox-$1-$ARCH-official.txz
	splash
	msg "$(gettext 'Done.')"
}

#### main
check_root
check_online
! yesno "\
$(gettext 'Running this program may replace your Firefox with the OFFICIAL binary
from Mozila, instead of Fatdog-build one.') $(gettext 'It should work, but in some
rare cases Fatdog may be missing some libraries that prevent this 
official from running.') $(gettext 'Are you sure you want to continue?')
" && exit

mkdir $WORKDIR; cd $WORKDIR

splash "$(gettext 'Checking latest released version ...')"
if ! LATEST=$(get_latest_version); then
    splash
    msg "$(gettext 'Cannot connect to Mozilla website. Leaving now.')"
    exit
fi
CURRENT=$(get_current_version)
splash
echo CURRENT=$CURRENT LATEST=$LATEST

if [ "$LATEST" = "$CURRENT" ]; then
    msg "$(eval echo `gettext 'You are already running the latest version $LATEST.'`)"
else
    if [ -z $CURRENT ]; then
        ! yesno "$(gettext 'You do not seem to have firefox installed.')
$(gettext 'Do you want to install it?')" && exit
    fi
   
    splash "$(eval echo `gettext 'Downloading firefox $LATEST, please wait ...'`) $(gettext 'Choose language.')"
    if wget -qO firefox.tar.bz2 "$(get_path)"; then
        package_firefox $LATEST
        install_firefox	$LATEST
    else
        splash
        msg "$(gettext 'Failed to download.')"
    fi
fi
cd - >/dev/null
exit
